/*
 *  node-arduino: Control your Arduino with Node
 *
 *  Copyright (c) 2010 Tobias Schneider
 *  node-arduino is freely distributable under the terms of the MIT license.
 */

#define SERIAL_BAUDRATE 115200

#define OPC_PIN_MODE         0x01
#define OPC_DIGITAL_READ     0x02
#define OPC_DIGITAL_WRITE    0x03
#define OPC_ANALOG_REFERENCE 0x04
#define OPC_ANALOG_READ      0x05
#define OPC_ANALOG_WRITE     0x06

long pinVal = 0;
long inpVal = 0;
long outVal = 0;

#define DDC_CAR_UP     0xE1
#define DDC_CAR_DN     0xE2
#define DDC_CAR_LT     0xE3
#define DDC_CAR_RT     0xE4
#define DDC_CAR_ST     0xE5
#define DDC_CAR_UL     0xE6
#define DDC_CAR_UR     0xE7
#define DDC_CAR_DL     0xE8
#define DDC_CAR_DR     0xE9
#define DDC_CAR_CT     0xF0

const int EN1 = 6;
const int EN2 = 5;
const int IN1 = 7;
const int IN2 = 4;

void setup() {
  Serial.begin(SERIAL_BAUDRATE);

  // Arduino Moter Shield
  pinMode(5, OUTPUT);
  pinMode(6, OUTPUT);
  pinMode(7, OUTPUT);
  pinMode(8, OUTPUT);
  
  // Head Lights
  pinMode(10, OUTPUT); // FRONT L
  pinMode(11, OUTPUT); // FRONT R
  pinMode(12, OUTPUT); // BACK  L
  pinMode(13, OUTPUT); // BACK  L
}

void handleMoter(int type, int pwn, bool reverse) {
  analogWrite( type == 1 ? EN1 : EN2, pwn);
  digitalWrite( type == 1 ? IN1 : IN2, reverse ? HIGH : LOW);
}

void loop() {
  pinVal = 0, inpVal = 0, outVal = 0;
  while (Serial.available() > 0) {
    switch (Serial.read()) {
      case OPC_PIN_MODE: {
        pinMode(Serial.read(), Serial.read());
        break;
      }
      case OPC_DIGITAL_READ: {
        delay(50);
        pinVal = Serial.read();
        inpVal = digitalRead(pinVal);
        outVal = pinVal << 16 | inpVal;
        Serial.println(outVal);
        break;
      }
      case OPC_DIGITAL_WRITE: {
        digitalWrite(Serial.read(), Serial.read());
        break;
      }
      case OPC_ANALOG_REFERENCE: {
        analogReference(Serial.read());
        break;
      }
      case OPC_ANALOG_READ: {
        delay(50);
        pinVal = Serial.read();
        inpVal = analogRead(pinVal);
        outVal = pinVal << 16 | inpVal;
        Serial.println(outVal);
        break;
      }
      case OPC_ANALOG_WRITE: {
        analogWrite(Serial.read(), Serial.read());
        break;
      }
      case DDC_CAR_UP: {
        handleMoter(1, 255, true);
//        handleMoter(2, 0, false);
        break;
      }      
/*
      case DDC_CAR_UL: {
        handleMoter(1, 255, true);
        handleMoter(2, 255, false);
        break;
      }
      case DDC_CAR_UR: {
        handleMoter(1, 255, true);
        handleMoter(2, 255, true);
        break;
      }*/
      case DDC_CAR_DN: {
        handleMoter(1, 255, false);
       // handleMoter(2, 0, false);
        break;
      }/*
      case DDC_CAR_DL: {
        handleMoter(1, 255, false);
        handleMoter(2, 255, false);
        break;
      }
      case DDC_CAR_DR: {
        handleMoter(1, 255, false);
        handleMoter(2, 255, true);
        break;
      }*/
      case DDC_CAR_LT: {
        handleMoter(2, 255, false);
        break;
      }
      case DDC_CAR_RT: {
        handleMoter(2, 255, true);
        break;
      }
      case DDC_CAR_ST: {
        handleMoter(1, 0, false);
//        handleMoter(2, 0, false);
        break;
      }
      case DDC_CAR_CT : {
        handleMoter(2, 0, false);
        break;
      }
    }
  }
}
